#include <SimpleModbusRtu_mega2560.h>
#include <Wire.h>

// comfileHMI 통신용
#define     comm_baudRate   57600   // 통신속도
#define     comm_slave      1       // 슬레이브 주소

//CSG 주소용
#define     csg_curTemp     0
#define     csg_minTemp     1 

// 레지스터 영역 의미구분
#define     isHeaterOn      (modbusCoil[0] & 0x01 ? HIGH : LOW)
#define     isLampOn        (modbusCoil[0] & 0x02 ? HIGH : LOW)
#define     isFood          (modbusCoil[0] & 0x04 ? HIGH : LOW)
#define     isChange        (modbusCoil[0] & 0x08 ? HIGH : LOW)
#define     isActFood       (modbusCoil[0] & 0x10 ? true : false)
#define     isActChange     (modbusCoil[0] & 0x20 ? true : false)

#define     setHeater       (modbusCoil[0] |= 0x01)
#define     clearHeater     (modbusCoil[0] &= ~0x01)

#define     clearActFood    (modbusCoil[0] &= ~0x10)
#define     clearActChange  (modbusCoil[0] &= ~0x20)

#define     getHeatMax      modbusRegister[HEATER_MAX]
#define     getHeatMin      modbusRegister[HEATER_MIN]
#define     HeatCurrent     modbusRegister[HEATER_CUR]


enum {HEATER=38,LAMP,ALRAM_FOOD,ALRAM_CHANGE,ACT_FOOD,ACT_CHANGE}     PIN_ALIAS;
enum {HEATER_MAX=0,HEATER_MIN,HEATER_CUR}                             HEATER_INDEX;

// CSG에서는 일반적인 FND처럼 각각의 LED를 알맞게 제어해야 숫자를 띄울 수 있습니다.
unsigned char     csg_lookupTable[10] = {0x3f,0x06,0x5b,0x4f,0x66,0x6d,0x7d,0x27,0x7f,0x6f};

//모드버스 메모리 영역
unsigned char modbusCoil[10];
unsigned short modbusRegister[10];

// 사용자 정의 함수 [선언부]
void csg_init(unsigned char channel);
void csg_print(unsigned char channel,unsigned short value);


void setup() {

  // 사용하는 포트설정
  pinMode(HEATER,OUTPUT);
  pinMode(LAMP,OUTPUT);
  pinMode(ALRAM_FOOD,OUTPUT);
  pinMode(ALRAM_CHANGE,OUTPUT);
  pinMode(ACT_FOOD,OUTPUT);
  pinMode(ACT_CHANGE,OUTPUT);

  // CSG 통신 설정
  Wire.begin();             // CSG는 I2C 통신을 사용합니다.
  Wire.setClock(50000);     // CSG와의 통신을 빠르게 진행하기위해 50KHz로 설정합니다.
  csg_init(csg_curTemp);
  csg_init(csg_minTemp);

  // 모드버스 통신 설정.
  usart1init(comm_baudRate);
  startSimpleModbus(comm_slave,modbusRegister,modbusCoil);
}

void loop() {

  //램프 조작
  digitalWrite(LAMP,isLampOn);
  digitalWrite(ALRAM_FOOD,isFood);
  digitalWrite(ALRAM_CHANGE,isChange);

  // 온도 측정.
  int measure_temp    =   analogRead(4);
  int val_heater_off  =   (getHeatMax - getHeatMin) / 2.0 + getHeatMin;   // 히터 동작지점 계산

  HeatCurrent = measure_temp;
  if(measure_temp <= getHeatMin)    // 히터 동작.
  {
    setHeater;
    digitalWrite(HEATER,HIGH);
  }
  else if(val_heater_off < measure_temp)  // 히터 동작 종료.
  {
    clearHeater;
    digitalWrite(HEATER,LOW);
  }

  // 급식 및 급수기 동작
  static int time_food_act      = 0;
  static int time_change_act    = 0;

  if(isActFood == true)   // 급식기 동작
  {
    ++time_food_act;
    digitalWrite(ACT_FOOD,HIGH);
    if((time_food_act %= 20) == 0)  // 급식기 동작종료.
    {
      digitalWrite(ACT_FOOD,LOW);
      clearActFood;
    }
  }

  if(isActChange == true)   // 급수기 동작
  {
    ++time_change_act;
    digitalWrite(ACT_CHANGE,HIGH);
    if((time_change_act %= 20) == 0)  // 급수기 동작종료.
    {
      digitalWrite(ACT_CHANGE,LOW);
      clearActChange;
    }
  }

  // csg 동작 실시.
  csg_print(csg_curTemp,HeatCurrent);
  csg_print(csg_minTemp,getHeatMin);
  
  delay(100);   // 임의적인 지연.
}

/* user-function declare */

void csg_init(unsigned char channel)
{
  Wire.beginTransmission(0x38+channel);       // CSG의 주소표현 : 0x38 + 접근주소
  Wire.write(0x00);   // 제어 레지스터 선택.
  Wire.write(0x47);   // 컨트롤 플레그 설정. [ 12mA출력 | 깜빡임 사용안함 | 모든자리 사용(다이나믹) ]
  Wire.endTransmission();
}
void csg_print(unsigned char channel,unsigned short value)
{
  Wire.beginTransmission(0x38+channel);       // CSG의 주소표현 : 0x38 + 접근주소
  Wire.write(0x01); // 1번째 자리부터 연속출력.
  Wire.write(csg_lookupTable[(value/1000)%10]);   // 1번째 자리 ( 1000의 자리 )
  Wire.write(csg_lookupTable[(value/100)%10]);    // 2번째 자리 ( 100의 자리)
  Wire.write(csg_lookupTable[(value/10)%10]);     // 3번째 자리 ( 10의 자리)
  Wire.write(csg_lookupTable[value%10]);          // 4번째 자리 ( 1의 자리)
  Wire.endTransmission();
}

